const vscode = require('vscode');

class LLMSelectorPanel {
    constructor(extensionUri, settingsManager) {
        this.extensionUri = extensionUri;
        this.panel = null;
        this.settingsManager = settingsManager;
    }

    async createOrShow(viewColumn) {
        if (this.panel) {
            this.panel.reveal(viewColumn);
        } else {
            this.panel = vscode.window.createWebviewPanel(
                'C4B3Rstudios.93m1n1gpt.llmSelector',
                '93m1n1GPT LLM Selector',
                viewColumn,
                {
                    enableScripts: true,
                    localResourceRoots: [vscode.Uri.joinPath(this.extensionUri, 'media')]
                }
            );

            this.panel.webview.html = await this.getWebviewContent(this.panel.webview);

            this.panel.webview.onDidReceiveMessage(
                async message => {
                    switch (message.command) {
                        case 'selectLLM':
                            await vscode.workspace.getConfiguration('93m1n1gpt').update(
                                'selectedModel',
                                message.model,
                                vscode.ConfigurationTarget.Global
                            );
                            vscode.window.showInformationMessage(`Selected LLM: ${message.model}`);
                            this.panel.dispose();
                            break;
                        case 'updateApiUrl':
                            await this.settingsManager.setSecure('apiUrl', message.url);
                            vscode.window.showInformationMessage('API URL updated securely');
                            break;
                    }
                }
            );

            this.panel.onDidDispose(() => {
                this.panel = null;
            });
        }
    }

    async getWebviewContent(webview) {
        try {
            const apiUrl = await this.settingsManager.getSecure('apiUrl') || 'http://localhost:11434';
            return `<!DOCTYPE html>
            <html lang="en">
            <head>
                <meta charset="UTF-8">
                <meta name="viewport" content="width=device-width, initial-scale=1.0">
                <title>93m1n1GPT LLM Selection</title>
                <style>
                    body {
                        padding: 20px;
                        font-family: var(--vscode-font-family);
                        color: var(--vscode-foreground);
                        line-height: 1.6;
                    }

                    .container {
                        max-width: 800px;
                        margin: 0 auto;
                    }

                    .card {
                        background: var(--vscode-editor-background);
                        border: 1px solid var(--vscode-panel-border);
                        border-radius: 6px;
                        padding: 20px;
                        margin-bottom: 20px;
                    }

                    h1 {
                        font-size: 24px;
                        font-weight: 600;
                        color: var(--vscode-foreground);
                        margin: 0 0 20px 0;
                        border-bottom: 1px solid var(--vscode-panel-border);
                        padding-bottom: 10px;
                    }

                    h2 {
                        font-size: 18px;
                        font-weight: 600;
                        color: var(--vscode-foreground);
                        margin: 20px 0 15px 0;
                    }

                    .input-group {
                        margin-bottom: 20px;
                    }

                    label {
                        display: block;
                        margin-bottom: 8px;
                        color: var(--vscode-foreground);
                        font-weight: 500;
                    }

                    input[type="text"], select {
                        width: 100%;
                        padding: 8px 12px;
                        background: var(--vscode-input-background);
                        color: var(--vscode-input-foreground);
                        border: 1px solid var(--vscode-input-border);
                        border-radius: 4px;
                        font-size: 14px;
                        margin-bottom: 10px;
                    }

                    input[type="text"]:focus, select:focus {
                        outline: none;
                        border-color: var(--vscode-focusBorder);
                    }

                    select {
                        height: 36px;
                    }

                    button {
                        background: var(--vscode-button-background);
                        color: var(--vscode-button-foreground);
                        border: none;
                        padding: 8px 16px;
                        border-radius: 4px;
                        font-size: 14px;
                        font-weight: 500;
                        cursor: pointer;
                        transition: background-color 0.2s;
                    }

                    button:hover {
                        background: var(--vscode-button-hoverBackground);
                    }

                    button:disabled {
                        opacity: 0.5;
                        cursor: not-allowed;
                    }

                    .status {
                        margin-top: 10px;
                        padding: 10px;
                        border-radius: 4px;
                        font-size: 14px;
                    }

                    .status.error {
                        background: var(--vscode-inputValidation-errorBackground);
                        border: 1px solid var(--vscode-inputValidation-errorBorder);
                        color: var(--vscode-inputValidation-errorForeground);
                    }

                    .status.info {
                        background: var(--vscode-infoBackground);
                        border: 1px solid var(--vscode-infoBorder);
                        color: var(--vscode-infoForeground);
                    }

                    .button-container {
                        display: flex;
                        gap: 10px;
                        margin-top: 15px;
                    }

                    .model-count {
                        font-size: 12px;
                        color: var(--vscode-descriptionForeground);
                        margin-top: 5px;
                    }

                    .spinner {
                        border: 2px solid var(--vscode-input-background);
                        border-top: 2px solid var(--vscode-focusBorder);
                        border-radius: 50%;
                        width: 16px;
                        height: 16px;
                        animation: spin 1s linear infinite;
                        display: inline-block;
                        vertical-align: middle;
                        margin-right: 8px;
                    }

                    @keyframes spin {
                        0% { transform: rotate(0deg); }
                        100% { transform: rotate(360deg); }
                    }
                </style>
            </head>
            <body>
                <div class="container">
                    <div class="card">
                        <h1>93m1n1GPT Configuration</h1>
                        
                        <div class="input-group">
                            <label for="apiUrl">Ollama API URL</label>
                            <input type="text" 
                                   id="apiUrl" 
                                   value="${apiUrl}" 
                                   placeholder="Enter Ollama API URL"
                            />
                            <button id="saveApiUrl">Save API URL</button>
                        </div>

                        <h2>LLM Model Selection</h2>
                        <div class="input-group">
                            <label for="llmSelector">Available Models</label>
                            <select id="llmSelector">
                                <option value="">Loading available models...</option>
                            </select>
                            <div class="model-count" id="modelCount"></div>
                        </div>

                        <div id="status" class="status" style="display: none;"></div>

                        <div class="button-container">
                            <button id="selectButton" disabled>Select Model</button>
                            <button id="refreshModels">
                                <span id="refreshSpinner" class="spinner" style="display: none;"></span>
                                Refresh Models
                            </button>
                        </div>
                    </div>
                </div>

                <script>
                    const vscode = acquireVsCodeApi();
                    const llmSelector = document.getElementById('llmSelector');
                    const selectButton = document.getElementById('selectButton');
                    const statusDiv = document.getElementById('status');
                    const apiUrlInput = document.getElementById('apiUrl');
                    const saveApiUrlButton = document.getElementById('saveApiUrl');
                    const refreshButton = document.getElementById('refreshModels');
                    const refreshSpinner = document.getElementById('refreshSpinner');
                    const modelCountDiv = document.getElementById('modelCount');

                    async function fetchAvailableLLMs() {
                        try {
                            refreshSpinner.style.display = 'inline-block';
                            const apiUrl = apiUrlInput.value;
                            const response = await fetch(apiUrl + '/api/tags');
                            if (!response.ok) {
                                throw new Error('Failed to fetch available LLMs');
                            }
                            const data = await response.json();
                            return data.models || [];
                        } catch (error) {
                            console.error('Error fetching available LLMs:', error);
                            showError('Failed to fetch models. Please ensure Ollama is running.');
                            return [];
                        } finally {
                            refreshSpinner.style.display = 'none';
                        }
                    }

                    function showError(message) {
                        statusDiv.style.display = 'block';
                        statusDiv.className = 'status error';
                        statusDiv.textContent = message;
                    }

                    function showStatus(message) {
                        statusDiv.style.display = 'block';
                        statusDiv.className = 'status info';
                        statusDiv.textContent = message;
                    }

                    async function populateLLMSelector() {
                        showStatus('Fetching available models...');
                        const llms = await fetchAvailableLLMs();
                        
                        llmSelector.innerHTML = '';
                        if (llms.length === 0) {
                            const option = document.createElement('option');
                            option.value = '';
                            option.textContent = 'No models found';
                            llmSelector.appendChild(option);
                            selectButton.disabled = true;
                            modelCountDiv.textContent = 'No models available';
                            return;
                        }

                        const defaultOption = document.createElement('option');
                        defaultOption.value = '';
                        defaultOption.textContent = 'Select a model';
                        llmSelector.appendChild(defaultOption);

                        llms.forEach(llm => {
                            const option = document.createElement('option');
                            option.value = llm.name;
                            option.textContent = llm.name;
                            llmSelector.appendChild(option);
                        });

                        modelCountDiv.textContent = \`\${llms.length} model\${llms.length === 1 ? '' : 's'} available\`;
                        selectButton.disabled = true;
                        statusDiv.style.display = 'none';
                    }

                    llmSelector.addEventListener('change', () => {
                        selectButton.disabled = !llmSelector.value;
                    });

                    selectButton.addEventListener('click', () => {
                        const selectedLLM = llmSelector.value;
                        if (selectedLLM) {
                            vscode.postMessage({
                                command: 'selectLLM',
                                model: selectedLLM
                            });
                        }
                    });

                    saveApiUrlButton.addEventListener('click', async () => {
                        showStatus('Updating API URL...');
                        await vscode.postMessage({
                            command: 'updateApiUrl',
                            url: apiUrlInput.value
                        });
                        populateLLMSelector();
                    });

                    refreshButton.addEventListener('click', () => {
                        populateLLMSelector();
                    });

                    // Initialize the selector
                    populateLLMSelector();
                </script>
            </body>
            </html>`;
        } catch (error) {
            console.error('Error generating webview content:', error);
            return `<html><body>Error loading content: ${error.message}</body></html>`;
        }
    }
}

module.exports = LLMSelectorPanel;