const vscode = require('vscode');

class SecureSettingsManager {
    constructor(context) {
        this.secretStorage = context.secrets;
        this.CONFIG_PREFIX = '93m1n1gpt.';
        this.SECURE_KEYS = ['apiUrl']; // List of keys that should be stored securely
    }

    async initialize() {
        // Migrate any existing settings to secure storage
        const config = vscode.workspace.getConfiguration('93m1n1gpt');
        for (const key of this.SECURE_KEYS) {
            const existingValue = config.get(key);
            if (existingValue) {
                await this.setSecure(key, existingValue);
                // Clear the value from regular configuration
                await config.update(key, undefined, vscode.ConfigurationTarget.Global);
            }
        }
    }

    async getSecure(key) {
        try {
            const value = await this.secretStorage.get(this.CONFIG_PREFIX + key);
            return value || null;
        } catch (error) {
            console.error(`Error retrieving secure setting ${key}:`, error);
            return null;
        }
    }

    async setSecure(key, value) {
        try {
            await this.secretStorage.store(this.CONFIG_PREFIX + key, value);
            return true;
        } catch (error) {
            console.error(`Error storing secure setting ${key}:`, error);
            return false;
        }
    }

    async deleteSecure(key) {
        try {
            await this.secretStorage.delete(this.CONFIG_PREFIX + key);
            return true;
        } catch (error) {
            console.error(`Error deleting secure setting ${key}:`, error);
            return false;
        }
    }

    // Helper method to get all settings (both secure and non-secure)
    async getAllSettings() {
        const config = vscode.workspace.getConfiguration('93m1n1gpt');
        const settings = {};

        // Get non-secure settings
        for (const key of Object.keys(config)) {
            if (!this.SECURE_KEYS.includes(key)) {
                settings[key] = config.get(key);
            }
        }

        // Get secure settings
        for (const key of this.SECURE_KEYS) {
            settings[key] = await this.getSecure(key);
        }

        return settings;
    }
}

module.exports = SecureSettingsManager;