// AISidebarProvider.js
const { MarkdownRenderer } = require("../utils/MarkdownRender");
const { ChatService } = require("../handlers/ChatService");
const { FileSystemManager } = require("./FileSystemManager");
const { GitManager } = require("./GitManager");
const { UIManager } = require("./UIManager");

class AISidebarProvider {
    constructor(extensionUri, settingsManager, repoTracker) {
        this._extensionUri = extensionUri;
        this._settingsManager = settingsManager;
        this.repoTracker = repoTracker;
        this.messageHistory = [];
        this._disposables = [];

        // Initialize services
        this.markdownRenderer = new MarkdownRenderer();
        this.chatService = new ChatService(settingsManager);
        this.fileManager = new FileSystemManager(repoTracker);
        this.gitManager = new GitManager(repoTracker);
        this.uiManager = new UIManager(this._extensionUri);
    }

    async resolveWebviewView(webviewView, context, _token) {
        this._view = webviewView;
        this.uiManager.initializeWebview(webviewView, this._extensionUri);
        
        await this._setupServices(webviewView);
        this._setupMessageHandlers(webviewView);
    }

    async _setupServices(webviewView) {
        await this.fileManager.setupFileWatcher(webviewView, this._disposables);
        await this.gitManager.setupGitWatcher(webviewView, this._disposables);
    }

    _setupMessageHandlers(webviewView) {
        webviewView.webview.onDidReceiveMessage(async (message) => {
            const handlers = {
                chat: () => this.chatService.handleChatMessage(webviewView, message.text, this.messageHistory, this.markdownRenderer, this.repoTracker),
                copyCode: () => this.uiManager.handleCopyCode(message.text),
                insertCode: () => this.uiManager.handleInsertCode(message.text),
                openFile: () => this.fileManager.openFileInEditor(message.path, message.line),
                getFileReferences: () => this.fileManager.handleFileReferenceRequest(webviewView, message.prefix),
                getGitStatus: () => this.gitManager.sendGitStatus(webviewView)
            };

            const handler = handlers[message.command];
            if (handler) {
                await handler();
            }
        });
    }

    dispose() {
        this._disposables.forEach(d => d.dispose());
    }
}

module.exports = { AISidebarProvider };