// FileSystemManager.js
const vscode = require("vscode");

class FileSystemManager {
    constructor(repoTracker) {
        this.repoTracker = repoTracker;
    }

    async setupFileWatcher(webviewView, disposables) {
        await this._sendFileTree(webviewView);
        
        const watcher = vscode.workspace.createFileSystemWatcher('**/*');
        ['onDidChange', 'onDidCreate', 'onDidDelete'].forEach(event => {
            watcher[event](() => this._sendFileTree(webviewView));
        });
        
        disposables.push(watcher);
    }

    async _sendFileTree(webviewView) {
        const fileTree = await this.repoTracker.buildFileTree();
        webviewView.webview.postMessage({
            command: 'updateFileTree',
            fileTree: fileTree
        });
    }

    async handleFileReferenceRequest(webviewView, prefix) {
        const fileTree = this.repoTracker.fileTree;
        const files = this._flattenFileTree(fileTree)
            .filter(file => file.path.toLowerCase().includes(prefix.toLowerCase()))
            .slice(0, 10);

        webviewView.webview.postMessage({
            command: 'fileReferenceSuggestions',
            suggestions: files
        });
    }

    _flattenFileTree(tree, path = '') {
        return tree.reduce((acc, item) => {
            if (item.type === 'file') {
                acc.push({ type: 'file', path: path + item.name, name: item.name });
            } else if (item.type === 'directory' && item.children) {
                acc.push(...this._flattenFileTree(item.children, path + item.name + '/'));
            }
            return acc;
        }, []);
    }

    async openFileInEditor(filePath, line) {
        try {
            const document = await vscode.workspace.openTextDocument(filePath);
            const editor = await vscode.window.showTextDocument(document);
            
            if (line) {
                const lineNumber = parseInt(line) - 1;
                const range = editor.document.lineAt(lineNumber).range;
                editor.selection = new vscode.Selection(range.start, range.end);
                editor.revealRange(range, vscode.TextEditorRevealType.InCenter);
            }
        } catch (error) {
            console.error('Error opening file:', error);
            vscode.window.showErrorMessage(`Failed to open file: ${filePath}`);
        }
    }
}

module.exports = { FileSystemManager };
