const vscode = require("vscode");
const { exec } = require('child_process');
const path = require('path')
const fs = require('fs')
const util = require('util');
const execPromise = util.promisify(exec);

class RepoTracker {
    constructor() {
        this.workspaceRoot = "";
        this.fileTree = [];
        this.gitInfo = null;
        this.ignoredPaths = [
            'node_modules',
            '.git',
            'dist',
            'build',
            '.next',
            '.cache'
        ];
        this._initializeCompletionProvider();
    }

    async initialize() {
        const workspaceFolders = vscode.workspace.workspaceFolders;
        if (!workspaceFolders) return;

        this.workspaceRoot = workspaceFolders[0].uri.fsPath;
        await this.buildFileTree();
        await this.initializeGitTracking();
    }

    async initializeGitTracking() {
        try {
            const { stdout: gitRoot } = await execPromise('git rev-parse --show-toplevel', {
                cwd: this.workspaceRoot
            });

            this.gitInfo = {
                root: gitRoot.trim(),
                branch: '',
                lastCommit: '',
                status: []
            };

            await this.updateGitStatus();
        } catch (error) {
            console.log('Not a git repository or git not installed: ', error);
            this.gitInfo = null;
        }
    }

    async updateGitStatus() {
        if (!this.gitInfo) return;

        try {
            // Get current branch
            const { stdout: branch } = await execPromise('git branch --show-current', {
                cwd: this.workspaceRoot
            });
            this.gitInfo.branch = branch.trim();

            // Get last commit
            const { stdout: lastCommit } = await execPromise('git log -1 --oneline', {
                cwd: this.workspaceRoot
            });
            this.gitInfo.lastCommit = lastCommit.trim();

            // Get status
            const { stdout: status } = await execPromise('git status --porcelain', {
                cwd: this.workspaceRoot
            });
            this.gitInfo.status = status.split('\n')
                .filter(line => line.trim())
                .map(line => ({
                    status: line.substring(0, 2).trim(),
                    file: line.substring(3).trim()
                }));
        } catch (error) {
            console.error('Error updating git status:', error);
        }
    }

    _initializeCompletionProvider() {
        // Register the @ completion provider
        vscode.languages.registerCompletionItemProvider(
            { pattern: '**' },
            {
                provideCompletionItems: (document, position) => {
                    const linePrefix = document.lineAt(position).text.substring(0, position.character);

                    // Check if we're typing after an @
                    if (!linePrefix.endsWith('@')) {
                        return undefined;
                    }

                    // Create completion items for all files in the workspace
                    return this.fileTree
                        .flat(Infinity)
                        .filter(item => item.type === 'file')
                        .map(file => {
                            const completionItem = new vscode.CompletionItem(
                                file.path,
                                vscode.CompletionItemKind.File
                            );

                            // When selected, it will insert the file path and prepare for line number
                            completionItem.insertText = file.path + ':';
                            completionItem.detail = `File: ${file.path}`;
                            completionItem.command = {
                                command: 'editor.action.triggerSuggest',
                                title: 'Re-trigger completions'
                            };

                            return completionItem;
                        });
                }
            },
            '@' // Trigger character
        );

        // Register the line number completion provider
        vscode.languages.registerCompletionItemProvider(
            { pattern: '**' },
            {
                provideCompletionItems: async (document, position) => {
                    const linePrefix = document.lineAt(position).text.substring(0, position.character);

                    // Check if we're typing after a file path and colon
                    const match = linePrefix.match(/@([^:]+):$/);
                    if (!match) {
                        return undefined;
                    }

                    const filePath = match[1];
                    const content = await this.getFileContent(filePath);
                    if (!content) {
                        return undefined;
                    }

                    // Create completion items for each line number with preview
                    const lines = content.split('\n');
                    return lines.map((line, index) => {
                        const lineNum = index + 1;
                        const completionItem = new vscode.CompletionItem(
                            `${lineNum}`,
                            vscode.CompletionItemKind.Reference
                        );

                        completionItem.detail = line.trim();
                        completionItem.documentation = new vscode.MarkdownString(
                            `Preview of line ${lineNum}:\n\`\`\`\n${line.trim()}\n\`\`\``
                        );

                        return completionItem;
                    });
                }
            },
            ':' // Trigger character
        );
    }

    async buildFileTree() {
        this.fileTree = await this.scanDirectory(this.workspaceRoot);
    }

    async scanDirectory(dirPath) {
        const entries = await fs.promises.readdir(dirPath, { withFileTypes: true });
        const files = [];

        for (const entry of entries) {
            const fullPath = path.join(dirPath, entry.name);
            const relativePath = path.relative(this.workspaceRoot, fullPath);

            if (this.ignoredPaths.some(ignored => relativePath.includes(ignored))) {
                continue;
            }

            if (entry.isDirectory()) {
                const subFiles = await this.scanDirectory(fullPath);
                files.push({
                    type: 'directory',
                    name: entry.name,
                    path: relativePath,
                    children: subFiles
                });
            } else {
                files.push({
                    type: 'file',
                    name: entry.name,
                    path: relativePath,
                    extension: path.extname(entry.name)
                });
            }
        }

        return files;
    }

    async getFileContent(filePath) {
        const fullPath = path.join(this.workspaceRoot, filePath);
        try {
            const content = await fs.promises.readFile(fullPath, 'utf-8');
            return content;
        } catch (error) {
            console.error(`Error reading file ${filePath}:`, error);
            return null;
        }
    }

    getFileTreeSummary() {
        return JSON.stringify(this.fileTree, null, 2);
    }
    getGitStatus() {
        return this.gitInfo;
    }

    async watchGitChanges() {
        // Set up a file system watcher for git changes
        const gitWatcher = vscode.workspace.createFileSystemWatcher(
            new vscode.RelativePattern(this.workspaceRoot, '.git/**')
        );

        gitWatcher.onDidChange(() => this.updateGitStatus());
        gitWatcher.onDidCreate(() => this.updateGitStatus());
        gitWatcher.onDidDelete(() => this.updateGitStatus());

        return gitWatcher;
    }
}

module.exports = { RepoTracker };